@@ -3,12 +3,12 @@
  * proc.c
  *	  routines to manage per-process shared memory data structure
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/storage/lmgr/proc.c,v 1.136 2003/10/16 20:59:35 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/storage/lmgr/proc.c,v 1.156 2004/12/31 22:01:05 pgsql Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -40,18 +40,19 @@
  */
 #include "postgres.h"
 
-#include <errno.h>
 #include <signal.h>
 #include <unistd.h>
 #include <sys/time.h>
 
 #include "miscadmin.h"
 #include "access/xact.h"
+#include "storage/bufmgr.h"
 #include "storage/ipc.h"
 #include "storage/proc.h"
 #include "storage/sinval.h"
 #include "storage/spin.h"
 
+
 /* GUC variables */
 int			DeadlockTimeout = 1000;
 int			StatementTimeout = 0;
@@ -66,11 +67,11 @@ PGPROC	   *MyProc = NULL;
  * relatively infrequently (only at backend startup or shutdown) and not for
  * very long, so a spinlock is okay.
  */
-static slock_t *ProcStructLock = NULL;
+NON_EXEC_STATIC slock_t *ProcStructLock = NULL;
 
+/* Pointers to shared-memory structures */
 static PROC_HDR *ProcGlobal = NULL;
-
-static PGPROC *DummyProc = NULL;
+static PGPROC *DummyProcs = NULL;
 
 static bool waitingForLock = false;
 static bool waitingForSignal = false;
@@ -83,19 +84,35 @@ static volatile bool deadlock_timeout_active = false;
 static struct timeval statement_fin_time;
 
 
-static void ProcKill(void);
-static void DummyProcKill(void);
+static void ProcKill(int code, Datum arg);
+static void DummyProcKill(int code, Datum arg);
 static bool CheckStatementTimeout(void);
 
 
+/*
+ * Report shared-memory space needed by InitProcGlobal.
+ */
+int
+ProcGlobalShmemSize(int maxBackends)
+{
+	int			size = 0;
+
+	size += MAXALIGN(sizeof(PROC_HDR)); /* ProcGlobal */
+	size += MAXALIGN(NUM_DUMMY_PROCS * sizeof(PGPROC));	/* DummyProcs */
+	size += MAXALIGN(maxBackends * sizeof(PGPROC));		/* MyProcs */
+	size += MAXALIGN(sizeof(slock_t)); /* ProcStructLock */
+
+	return size;
+}
+
 /*
  * Report number of semaphores needed by InitProcGlobal.
  */
 int
 ProcGlobalSemas(int maxBackends)
 {
-	/* We need a sema per backend, plus one for the dummy process. */
-	return maxBackends + 1;
+	/* We need a sema per backend, plus one for each dummy process. */
+	return maxBackends + NUM_DUMMY_PROCS;
 }
 
 /*
@@ -121,20 +138,32 @@ ProcGlobalSemas(int maxBackends)
 void
 InitProcGlobal(int maxBackends)
 {
-	bool		found = false;
+	bool		foundProcGlobal,
+				foundDummy;
 
 	/* Create or attach to the ProcGlobal shared structure */
 	ProcGlobal = (PROC_HDR *)
-		ShmemInitStruct("Proc Header", sizeof(PROC_HDR), &found);
+		ShmemInitStruct("Proc Header", sizeof(PROC_HDR), &foundProcGlobal);
 
-	/* --------------------
-	 * We're the first - initialize.
-	 * XXX if found should ever be true, it is a sign of impending doom ...
-	 * ought to complain if so?
-	 * --------------------
+	/*
+	 * Create or attach to the PGPROC structures for dummy (checkpoint)
+	 * processes, too.	These do not get linked into the freeProcs list.
 	 */
-	if (!found)
+	DummyProcs = (PGPROC *)
+		ShmemInitStruct("DummyProcs", NUM_DUMMY_PROCS * sizeof(PGPROC),
+						&foundDummy);
+
+	if (foundProcGlobal || foundDummy)
 	{
+		/* both should be present or neither */
+		Assert(foundProcGlobal && foundDummy);
+	}
+	else
+	{
+		/*
+		 * We're the first - initialize.
+		 */
+		PGPROC	   *procs;
 		int			i;
 
 		ProcGlobal->freeProcs = INVALID_OFFSET;
@@ -143,34 +172,25 @@ InitProcGlobal(int maxBackends)
 		 * Pre-create the PGPROC structures and create a semaphore for
 		 * each.
 		 */
-		for (i = 0; i < maxBackends; i++)
-		{
-			PGPROC	   *proc;
-
-			proc = (PGPROC *) ShmemAlloc(sizeof(PGPROC));
-			if (!proc)
-				ereport(FATAL,
-						(errcode(ERRCODE_OUT_OF_MEMORY),
-						 errmsg("out of shared memory")));
-			MemSet(proc, 0, sizeof(PGPROC));
-			PGSemaphoreCreate(&proc->sem);
-			proc->links.next = ProcGlobal->freeProcs;
-			ProcGlobal->freeProcs = MAKE_OFFSET(proc);
-		}
-
-		/*
-		 * Pre-allocate a PGPROC structure for dummy (checkpoint)
-		 * processes, too.	This does not get linked into the freeProcs
-		 * list.
-		 */
-		DummyProc = (PGPROC *) ShmemAlloc(sizeof(PGPROC));
-		if (!DummyProc)
+		procs = (PGPROC *) ShmemAlloc(maxBackends * sizeof(PGPROC));
+		if (!procs)
 			ereport(FATAL,
 					(errcode(ERRCODE_OUT_OF_MEMORY),
 					 errmsg("out of shared memory")));
-		MemSet(DummyProc, 0, sizeof(PGPROC));
-		DummyProc->pid = 0;		/* marks DummyProc as not in use */
-		PGSemaphoreCreate(&DummyProc->sem);
+		MemSet(procs, 0, maxBackends * sizeof(PGPROC));
+		for (i = 0; i < maxBackends; i++)
+		{
+			PGSemaphoreCreate(&(procs[i].sem));
+			procs[i].links.next = ProcGlobal->freeProcs;
+			ProcGlobal->freeProcs = MAKE_OFFSET(&procs[i]);
+		}
+
+		MemSet(DummyProcs, 0, NUM_DUMMY_PROCS * sizeof(PGPROC));
+		for (i = 0; i < NUM_DUMMY_PROCS; i++)
+		{
+			DummyProcs[i].pid = 0;		/* marks dummy proc as not in use */
+			PGSemaphoreCreate(&(DummyProcs[i].sem));
+		}
 
 		/* Create ProcStructLock spinlock, too */
 		ProcStructLock = (slock_t *) ShmemAlloc(sizeof(slock_t));
@@ -231,7 +251,7 @@ InitProcess(void)
 	 * prepared for us by InitProcGlobal.
 	 */
 	SHMQueueElemInit(&(MyProc->links));
-	MyProc->errType = STATUS_OK;
+	MyProc->waitStatus = STATUS_OK;
 	MyProc->xid = InvalidTransactionId;
 	MyProc->xmin = InvalidTransactionId;
 	MyProc->pid = MyProcPid;
@@ -241,8 +261,8 @@ InitProcess(void)
 	MyProc->lwExclusive = false;
 	MyProc->lwWaitLink = NULL;
 	MyProc->waitLock = NULL;
-	MyProc->waitHolder = NULL;
-	SHMQueueInit(&(MyProc->procHolders));
+	MyProc->waitProcLock = NULL;
+	SHMQueueInit(&(MyProc->procLocks));
 
 	/*
 	 * Arrange to clean up at backend exit.
@@ -268,34 +288,44 @@ InitProcess(void)
  * This is called by checkpoint processes so that they will have a MyProc
  * value that's real enough to let them wait for LWLocks.  The PGPROC and
  * sema that are assigned are the extra ones created during InitProcGlobal.
+ *
+ * Dummy processes are presently not expected to wait for real (lockmgr)
+ * locks, nor to participate in sinval messaging.
  */
 void
-InitDummyProcess(void)
+InitDummyProcess(int proctype)
 {
+	PGPROC	   *dummyproc;
+
 	/*
 	 * ProcGlobal should be set by a previous call to InitProcGlobal (we
 	 * inherit this by fork() from the postmaster).
 	 */
-	if (ProcGlobal == NULL || DummyProc == NULL)
+	if (ProcGlobal == NULL || DummyProcs == NULL)
 		elog(PANIC, "proc header uninitialized");
 
 	if (MyProc != NULL)
 		elog(ERROR, "you already exist");
 
+	Assert(proctype >= 0 && proctype < NUM_DUMMY_PROCS);
+
+	dummyproc = &DummyProcs[proctype];
+
 	/*
-	 * DummyProc should not presently be in use by anyone else
+	 * dummyproc should not presently be in use by anyone else
 	 */
-	if (DummyProc->pid != 0)
-		elog(FATAL, "DummyProc is in use by PID %d", DummyProc->pid);
-	MyProc = DummyProc;
+	if (dummyproc->pid != 0)
+		elog(FATAL, "DummyProc[%d] is in use by PID %d",
+			 proctype, dummyproc->pid);
+	MyProc = dummyproc;
 
 	/*
 	 * Initialize all fields of MyProc, except MyProc->sem which was set
 	 * up by InitProcGlobal.
 	 */
-	MyProc->pid = MyProcPid;	/* marks DummyProc as in use by me */
+	MyProc->pid = MyProcPid;	/* marks dummy proc as in use by me */
 	SHMQueueElemInit(&(MyProc->links));
-	MyProc->errType = STATUS_OK;
+	MyProc->waitStatus = STATUS_OK;
 	MyProc->xid = InvalidTransactionId;
 	MyProc->xmin = InvalidTransactionId;
 	MyProc->databaseId = MyDatabaseId;
@@ -304,13 +334,13 @@ InitDummyProcess(void)
 	MyProc->lwExclusive = false;
 	MyProc->lwWaitLink = NULL;
 	MyProc->waitLock = NULL;
-	MyProc->waitHolder = NULL;
-	SHMQueueInit(&(MyProc->procHolders));
+	MyProc->waitProcLock = NULL;
+	SHMQueueInit(&(MyProc->procLocks));
 
 	/*
 	 * Arrange to clean up at process exit.
 	 */
-	on_shmem_exit(DummyProcKill, 0);
+	on_shmem_exit(DummyProcKill, Int32GetDatum(proctype));
 
 	/*
 	 * We might be reusing a semaphore that belonged to a failed process.
@@ -335,15 +365,32 @@ LockWaitCancel(void)
 	if (!waitingForLock)
 		return false;
 
-	waitingForLock = false;
-
 	/* Turn off the deadlock timer, if it's still running (see ProcSleep) */
 	disable_sig_alarm(false);
 
 	/* Unlink myself from the wait queue, if on it (might not be anymore!) */
 	LWLockAcquire(LockMgrLock, LW_EXCLUSIVE);
+
 	if (MyProc->links.next != INVALID_OFFSET)
+	{
+		/* We could not have been granted the lock yet */
+		Assert(MyProc->waitStatus == STATUS_ERROR);
 		RemoveFromWaitQueue(MyProc);
+	}
+	else
+	{
+		/*
+		 * Somebody kicked us off the lock queue already.  Perhaps they
+		 * granted us the lock, or perhaps they detected a deadlock. If
+		 * they did grant us the lock, we'd better remember it in our
+		 * local lock table.
+		 */
+		if (MyProc->waitStatus == STATUS_OK)
+			GrantAwaitedLock();
+	}
+
+	waitingForLock = false;
+
 	LWLockRelease(LockMgrLock);
 
 	/*
@@ -367,15 +414,19 @@ LockWaitCancel(void)
 
 /*
  * ProcReleaseLocks() -- release locks associated with current transaction
- *			at transaction commit or abort
+ *			at main transaction commit or abort
  *
- * At commit, we release only locks tagged with the current transaction's XID,
- * leaving those marked with XID 0 (ie, session locks) undisturbed.  At abort,
- * we release all locks including XID 0, because we need to clean up after
- * a failure.  This logic will need extension if we ever support nested
- * transactions.
+ * At main transaction commit, we release all locks except session locks.
+ * At main transaction abort, we release all locks including session locks;
+ * this lets us clean up after a VACUUM FULL failure.
  *
- * Note that user locks are not released in either case.
+ * At subtransaction commit, we don't release any locks (so this func is not
+ * needed at all); we will defer the releasing to the parent transaction.
+ * At subtransaction abort, we release all locks held by the subtransaction;
+ * this is implemented by retail releasing of the locks under control of
+ * the ResourceOwner mechanism.
+ *
+ * Note that user locks are not released in any case.
  */
 void
 ProcReleaseLocks(bool isCommit)
@@ -385,8 +436,7 @@ ProcReleaseLocks(bool isCommit)
 	/* If waiting, get off wait queue (should only be needed after error) */
 	LockWaitCancel();
 	/* Release locks */
-	LockReleaseAll(DEFAULT_LOCKMETHOD, MyProc,
-				   !isCommit, GetCurrentTransactionId());
+	LockReleaseAll(DEFAULT_LOCKMETHOD, !isCommit);
 }
 
 
@@ -395,7 +445,7 @@ ProcReleaseLocks(bool isCommit)
  *		this process. Release any of its held LW locks.
  */
 static void
-ProcKill(void)
+ProcKill(int code, Datum arg)
 {
 	/* use volatile pointer to prevent code rearrangement */
 	volatile PROC_HDR *procglobal = ProcGlobal;
@@ -411,19 +461,17 @@ ProcKill(void)
 	 * shutdown callback registered by the bufmgr ... but we must do this
 	 * *after* LWLockReleaseAll and *before* zapping MyProc.
 	 */
-	AbortBufferIO();
-	UnlockBuffers();
-	AtEOXact_Buffers(false);
+	AtProcExit_Buffers();
 
 	/* Get off any wait queue I might be on */
 	LockWaitCancel();
 
 	/* Remove from the standard lock table */
-	LockReleaseAll(DEFAULT_LOCKMETHOD, MyProc, true, InvalidTransactionId);
+	LockReleaseAll(DEFAULT_LOCKMETHOD, true);
 
 #ifdef USER_LOCKS
 	/* Remove from the user lock table */
-	LockReleaseAll(USER_LOCKMETHOD, MyProc, true, InvalidTransactionId);
+	LockReleaseAll(USER_LOCKMETHOD, true);
 #endif
 
 	SpinLockAcquire(ProcStructLock);
@@ -444,21 +492,26 @@ ProcKill(void)
  *		as not-in-use.
  */
 static void
-DummyProcKill(void)
+DummyProcKill(int code, Datum arg)
 {
-	Assert(MyProc != NULL && MyProc == DummyProc);
+	int			proctype = DatumGetInt32(arg);
+	PGPROC	   *dummyproc;
+
+	Assert(proctype >= 0 && proctype < NUM_DUMMY_PROCS);
+
+	dummyproc = &DummyProcs[proctype];
+
+	Assert(MyProc == dummyproc);
 
 	/* Release any LW locks I am holding */
 	LWLockReleaseAll();
 
 	/* Release buffer locks and pins, too */
-	AbortBufferIO();
-	UnlockBuffers();
-	AtEOXact_Buffers(false);
+	AtProcExit_Buffers();
 
 	/* I can't be on regular lock queues, so needn't check */
 
-	/* Mark DummyProc no longer in use */
+	/* Mark dummy proc no longer in use */
 	MyProc->pid = 0;
 
 	/* PGPROC struct isn't mine anymore */
@@ -524,14 +577,14 @@ ProcQueueInit(PROC_QUEUE *queue)
  * semaphore is normally zero, so when we try to acquire it, we sleep.
  */
 int
-ProcSleep(LOCKMETHODTABLE *lockMethodTable,
+ProcSleep(LockMethod lockMethodTable,
 		  LOCKMODE lockmode,
 		  LOCK *lock,
 		  PROCLOCK *proclock)
 {
 	LWLockId	masterLock = lockMethodTable->masterLock;
 	PROC_QUEUE *waitQueue = &(lock->waitProcs);
-	int			myHeldLocks = MyProc->heldLocks;
+	LOCKMASK	myHeldLocks = MyProc->heldLocks;
 	bool		early_deadlock = false;
 	PGPROC	   *proc;
 	int			i;
@@ -556,7 +609,7 @@ ProcSleep(LOCKMETHODTABLE *lockMethodTable,
 	 */
 	if (myHeldLocks != 0)
 	{
-		int			aheadRequests = 0;
+		LOCKMASK	aheadRequests = 0;
 
 		proc = (PGPROC *) MAKE_PTR(waitQueue->links.next);
 		for (i = 0; i < waitQueue->size; i++)
@@ -590,13 +643,14 @@ ProcSleep(LOCKMETHODTABLE *lockMethodTable,
 				{
 					/* Skip the wait and just grant myself the lock. */
 					GrantLock(lock, proclock, lockmode);
+					GrantAwaitedLock();
 					return STATUS_OK;
 				}
 				/* Break out of loop to put myself before him */
 				break;
 			}
 			/* Nope, so advance to next waiter */
-			aheadRequests |= (1 << proc->waitLockMode);
+			aheadRequests |= LOCKBIT_ON(proc->waitLockMode);
 			proc = (PGPROC *) MAKE_PTR(proc->links.next);
 		}
 
@@ -618,14 +672,14 @@ ProcSleep(LOCKMETHODTABLE *lockMethodTable,
 	SHMQueueInsertBefore(&(proc->links), &(MyProc->links));
 	waitQueue->size++;
 
-	lock->waitMask |= (1 << lockmode);
+	lock->waitMask |= LOCKBIT_ON(lockmode);
 
 	/* Set up wait information in PGPROC object, too */
 	MyProc->waitLock = lock;
-	MyProc->waitHolder = proclock;
+	MyProc->waitProcLock = proclock;
 	MyProc->waitLockMode = lockmode;
 
-	MyProc->errType = STATUS_OK;	/* initialize result for success */
+	MyProc->waitStatus = STATUS_ERROR;	/* initialize result for error */
 
 	/*
 	 * If we detected deadlock, give up without waiting.  This must agree
@@ -635,7 +689,6 @@ ProcSleep(LOCKMETHODTABLE *lockMethodTable,
 	if (early_deadlock)
 	{
 		RemoveFromWaitQueue(MyProc);
-		MyProc->errType = STATUS_ERROR;
 		return STATUS_ERROR;
 	}
 
@@ -655,8 +708,8 @@ ProcSleep(LOCKMETHODTABLE *lockMethodTable,
 	/*
 	 * Set timer so we can wake up after awhile and check for a deadlock.
 	 * If a deadlock is detected, the handler releases the process's
-	 * semaphore and sets MyProc->errType = STATUS_ERROR, allowing us to
-	 * know that we must report failure rather than success.
+	 * semaphore and sets MyProc->waitStatus = STATUS_ERROR, allowing us
+	 * to know that we must report failure rather than success.
 	 *
 	 * By delaying the check until we've waited for a bit, we can avoid
 	 * running the rather expensive deadlock-check code in most cases.
@@ -675,8 +728,10 @@ ProcSleep(LOCKMETHODTABLE *lockMethodTable,
 	 * We pass interruptOK = true, which eliminates a window in which
 	 * cancel/die interrupts would be held off undesirably.  This is a
 	 * promise that we don't mind losing control to a cancel/die interrupt
-	 * here.  We don't, because we have no state-change work to do after
-	 * being granted the lock (the grantor did it all).
+	 * here.  We don't, because we have no shared-state-change work to do
+	 * after being granted the lock (the grantor did it all).  We do have
+	 * to worry about updating the locallock table, but if we lose control
+	 * to an error, LockWaitCancel will fix that up.
 	 */
 	PGSemaphoreLock(&MyProc->sem, true);
 
@@ -687,20 +742,28 @@ ProcSleep(LOCKMETHODTABLE *lockMethodTable,
 		elog(FATAL, "could not disable timer for process wakeup");
 
 	/*
-	 * Now there is nothing for LockWaitCancel to do.
+	 * Re-acquire the locktable's masterLock.  We have to do this to hold
+	 * off cancel/die interrupts before we can mess with waitingForLock
+	 * (else we might have a missed or duplicated locallock update).
+	 */
+	LWLockAcquire(masterLock, LW_EXCLUSIVE);
+
+	/*
+	 * We no longer want LockWaitCancel to do anything.
 	 */
 	waitingForLock = false;
 
 	/*
-	 * Re-acquire the locktable's masterLock.
+	 * If we got the lock, be sure to remember it in the locallock table.
 	 */
-	LWLockAcquire(masterLock, LW_EXCLUSIVE);
+	if (MyProc->waitStatus == STATUS_OK)
+		GrantAwaitedLock();
 
 	/*
 	 * We don't have to do anything else, because the awaker did all the
 	 * necessary update of the lock table and MyProc.
 	 */
-	return MyProc->errType;
+	return MyProc->waitStatus;
 }
 
 
@@ -715,7 +778,7 @@ ProcSleep(LOCKMETHODTABLE *lockMethodTable,
  * to twiddle the lock's request counts too --- see RemoveFromWaitQueue.
  */
 PGPROC *
-ProcWakeup(PGPROC *proc, int errType)
+ProcWakeup(PGPROC *proc, int waitStatus)
 {
 	PGPROC	   *retProc;
 
@@ -724,7 +787,7 @@ ProcWakeup(PGPROC *proc, int errType)
 	/* Proc should be sleeping ... */
 	if (proc->links.prev == INVALID_OFFSET ||
 		proc->links.next == INVALID_OFFSET)
-		return (PGPROC *) NULL;
+		return NULL;
 
 	/* Save next process before we zap the list link */
 	retProc = (PGPROC *) MAKE_PTR(proc->links.next);
@@ -735,8 +798,8 @@ ProcWakeup(PGPROC *proc, int errType)
 
 	/* Clean up process' state and pass it the ok/fail signal */
 	proc->waitLock = NULL;
-	proc->waitHolder = NULL;
-	proc->errType = errType;
+	proc->waitProcLock = NULL;
+	proc->waitStatus = waitStatus;
 
 	/* And awaken it */
 	PGSemaphoreUnlock(&proc->sem);
@@ -750,12 +813,12 @@ ProcWakeup(PGPROC *proc, int errType)
  *		for lock, waken any that are no longer blocked.
  */
 void
-ProcLockWakeup(LOCKMETHODTABLE *lockMethodTable, LOCK *lock)
+ProcLockWakeup(LockMethod lockMethodTable, LOCK *lock)
 {
 	PROC_QUEUE *waitQueue = &(lock->waitProcs);
 	int			queue_size = waitQueue->size;
 	PGPROC	   *proc;
-	int			aheadRequests = 0;
+	LOCKMASK	aheadRequests = 0;
 
 	Assert(queue_size >= 0);
 
@@ -776,12 +839,12 @@ ProcLockWakeup(LOCKMETHODTABLE *lockMethodTable, LOCK *lock)
 			LockCheckConflicts(lockMethodTable,
 							   lockmode,
 							   lock,
-							   proc->waitHolder,
+							   proc->waitProcLock,
 							   proc,
 							   NULL) == STATUS_OK)
 		{
 			/* OK to waken */
-			GrantLock(lock, proc->waitHolder, lockmode);
+			GrantLock(lock, proc->waitProcLock, lockmode);
 			proc = ProcWakeup(proc, STATUS_OK);
 
 			/*
@@ -796,7 +859,7 @@ ProcLockWakeup(LOCKMETHODTABLE *lockMethodTable, LOCK *lock)
 			 * Cannot wake this guy. Remember his request for later
 			 * checks.
 			 */
-			aheadRequests |= (1 << lockmode);
+			aheadRequests |= LOCKBIT_ON(lockmode);
 			proc = (PGPROC *) MAKE_PTR(proc->links.next);
 		}
 	}
@@ -863,10 +926,10 @@ CheckDeadLock(void)
 	RemoveFromWaitQueue(MyProc);
 
 	/*
-	 * Set MyProc->errType to STATUS_ERROR so that ProcSleep will report
-	 * an error after we return from the signal handler.
+	 * Set MyProc->waitStatus to STATUS_ERROR so that ProcSleep will
+	 * report an error after we return from the signal handler.
 	 */
-	MyProc->errType = STATUS_ERROR;
+	MyProc->waitStatus = STATUS_ERROR;
 
 	/*
 	 * Unlock my semaphore so that the interrupted ProcSleep() call can
@@ -951,9 +1014,6 @@ ProcSendSignal(BackendId procId)
 bool
 enable_sig_alarm(int delayms, bool is_statement_timeout)
 {
-#ifdef WIN32
-#warning add Win32 timer
-#else
 	struct timeval fin_time;
 
 #ifndef __BEOS__
@@ -1023,7 +1083,6 @@ enable_sig_alarm(int delayms, bool is_statement_timeout)
 	time_interval = delayms * 1000;		/* usecs */
 	if (set_alarm(time_interval, B_ONE_SHOT_RELATIVE_ALARM) < 0)
 		return false;
-#endif
 #endif
 	return true;
 }
@@ -1038,10 +1097,6 @@ enable_sig_alarm(int delayms, bool is_statement_timeout)
 bool
 disable_sig_alarm(bool is_statement_timeout)
 {
-#ifdef WIN32
-#warning add Win32 timer
-#else
-
 	/*
 	 * Always disable the interrupt if it is active; this avoids being
 	 * interrupted by the signal handler and thereby possibly getting
@@ -1081,7 +1136,6 @@ disable_sig_alarm(bool is_statement_timeout)
 		if (!CheckStatementTimeout())
 			return false;
 	}
-#endif
 	return true;
 }
 
@@ -1114,9 +1168,6 @@ CheckStatementTimeout(void)
 	else
 	{
 		/* Not time yet, so (re)schedule the interrupt */
-#ifdef WIN32
-#warning add win32 timer
-#else
 #ifndef __BEOS__
 		struct itimerval timeval;
 
@@ -1139,7 +1190,6 @@ CheckStatementTimeout(void)
 			(statement_fin_time.tv_usec - now.tv_usec);
 		if (set_alarm(time_interval, B_ONE_SHOT_RELATIVE_ALARM) < 0)
 			return false;
-#endif
 #endif
 	}
 
