@@ -2,9 +2,9 @@
  *
  * PostgreSQL locale utilities
  *
- * Portions Copyright (c) 2002-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 2002-2005, PostgreSQL Global Development Group
  *
- * $Header: /cvsroot/pgsql/src/backend/utils/adt/pg_locale.c,v 1.23 2003/08/04 23:59:38 tgl Exp $
+ * $PostgreSQL: pgsql/src/backend/utils/adt/pg_locale.c,v 1.30.4.1 2005/03/16 00:03:02 momjian Exp $
  *
  *-----------------------------------------------------------------------
  */
@@ -73,7 +73,7 @@ char	   *locale_time;
  * valid.  (See explanation at the top of this file.)
  */
 static const char *
-locale_xxx_assign(int category, const char *value, bool doit, bool interactive)
+locale_xxx_assign(int category, const char *value, bool doit, GucSource source)
 {
 	char	   *save;
 
@@ -99,21 +99,21 @@ locale_xxx_assign(int category, const char *value, bool doit, bool interactive)
 
 
 const char *
-locale_monetary_assign(const char *value, bool doit, bool interactive)
+locale_monetary_assign(const char *value, bool doit, GucSource source)
 {
-	return locale_xxx_assign(LC_MONETARY, value, doit, interactive);
+	return locale_xxx_assign(LC_MONETARY, value, doit, source);
 }
 
 const char *
-locale_numeric_assign(const char *value, bool doit, bool interactive)
+locale_numeric_assign(const char *value, bool doit, GucSource source)
 {
-	return locale_xxx_assign(LC_NUMERIC, value, doit, interactive);
+	return locale_xxx_assign(LC_NUMERIC, value, doit, source);
 }
 
 const char *
-locale_time_assign(const char *value, bool doit, bool interactive)
+locale_time_assign(const char *value, bool doit, GucSource source)
 {
-	return locale_xxx_assign(LC_TIME, value, doit, interactive);
+	return locale_xxx_assign(LC_TIME, value, doit, source);
 }
 
 
@@ -121,8 +121,9 @@ locale_time_assign(const char *value, bool doit, bool interactive)
  * We allow LC_MESSAGES to actually be set globally.
  */
 const char *
-locale_messages_assign(const char *value, bool doit, bool interactive)
+locale_messages_assign(const char *value, bool doit, GucSource source)
 {
+#ifndef WIN32
 	/*
 	 * LC_MESSAGES category does not exist everywhere, but accept it
 	 * anyway
@@ -134,9 +135,37 @@ locale_messages_assign(const char *value, bool doit, bool interactive)
 			return NULL;
 	}
 	else
-		value = locale_xxx_assign(LC_MESSAGES, value, false, interactive);
-#endif
+		value = locale_xxx_assign(LC_MESSAGES, value, false, source);
+#endif   /* LC_MESSAGES */
 	return value;
+
+#else /* WIN32 */
+
+	/*
+	 * Win32 does not have working setlocale() for LC_MESSAGES. We can only
+	 * use environment variables to change it (per gettext FAQ).  This
+	 * means we can't actually check the supplied value, so always assume
+	 * it's good.  Also, ignore attempts to set to "", which really means
+	 * "keep using the old value".  (Actually it means "use the environment
+	 * value", but we are too lazy to try to implement that exactly.)
+	 */
+	if (doit && value[0])
+	{
+		/*
+		 * We need to modify both the process environment and the cached
+		 * version in msvcrt
+		 */
+		static char env[128];
+
+		if (!SetEnvironmentVariable("LC_MESSAGES", value))
+			return NULL;
+
+		snprintf(env, sizeof(env)-1, "LC_MESSAGES=%s", value);
+		if (_putenv(env))
+			return NULL;
+	}
+	return value;
+#endif /* WIN32 */
 }
 
 
@@ -167,6 +196,33 @@ lc_collate_is_c(void)
 }
 
 
+/*
+ * We'd like to cache whether LC_CTYPE is C (or POSIX), so we can
+ * optimize a few code paths in various places.
+ */
+bool
+lc_ctype_is_c(void)
+{
+	/* Cache result so we only have to compute it once */
+	static int	result = -1;
+	char	   *localeptr;
+
+	if (result >= 0)
+		return (bool) result;
+	localeptr = setlocale(LC_CTYPE, NULL);
+	if (!localeptr)
+		elog(ERROR, "invalid LC_CTYPE setting");
+
+	if (strcmp(localeptr, "C") == 0)
+		result = true;
+	else if (strcmp(localeptr, "POSIX") == 0)
+		result = true;
+	else
+		result = false;
+	return (bool) result;
+}
+
+
 /*
  * Frees the malloced content of a struct lconv.  (But not the struct
  * itself.)
